/*
** Module   : ErrorWriterBatch.java
** Abstract : Implementation for error writer for batch mode.
**
** Copyright (c) 2013-2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 EVL 20131119 Created initial version.
** 002 EVL 20140115 Adding background and foreground differences support.
** 003 EVL 20140210 In Windows we need to use println() to display message in background mode.
** 004 EVL 20140221 If STDOUT is redirected by OS command we need to use println(). In Linux we
**                  have to also use println(). 
*/

package com.goldencode.p2j.ui;

import com.goldencode.p2j.util.*;
import com.goldencode.util.*;

/**
 * Implements display outputs for batch mode.
 */
public class ErrorWriterBatch
implements ErrorWriter
{
   /** Client exports access. */
   private ClientExports client = null; 
   
   /**
    * The only constructor used to create error writer.
    *
    * @param    client
    *           The client instance associated with writer.
    */
   public ErrorWriterBatch(ClientExports client)
   {
      this.client = client;
   }
   
   /**
    * Displays error message in bach mode.
    *
    * @param    errmsg
    *           Text of the message to display.
    */
   public void displayError(String errmsg)
   {
      if (PlatformHelper.isUnderWindowsFamily())
      {
         if (isBatchInBackground())
         {
            System.out.println(errmsg);
         }
         else
         {
            // TODO: In Windows the native message box should be displayed here.
            System.out.print(errmsg);
         }
      }
      else
      {
         System.out.println(errmsg);
      }
   }
   
   /**
    * Displays error message using currently installed writer implementation.  This is the
    * redirected version of the displayError.  If the output is redirected to file the message
    * is writing to the file.
    *
    * @param    errmsg
    *           Text of the message to display.
    */
   public void displayErrorRedirected(String errmsg)
   {
      if (client != null && client.isRedirected())
      {
         // we are running on the client
         ColorSpec cs = new ColorSpec(ColorSpec.COLOR_MESSAGES);
         
         client.message(errmsg, false, null, false, null, cs.convert());
      }
      else
      {
         displayError(errmsg);
      }
   }
    
   /**
    * Checks if the client is running in batch mode.
    * 
    * @return  <code>true</code> for batch mode, <code>false</code> otherwise.
    * 
    */
   public boolean isInBatchMode()
   {
      return (client != null) ? client.isInBatchMode() : false;
   }

   /**
    * Checks if the client is running in batch background mode.
    * 
    * @return  <code>true</code> for background mode, <code>false</code> otherwise.
    * 
    */
   public boolean isBatchInBackground()
   {
      return (client != null) ? client.isBatchInBackground() : false;
   }

   /**
    * Displays error message in bach mode.
    *
    * @param    errmsg
    *           Text of the message to display.
    */
   public void message(String errmsg)
   {
      if (PlatformHelper.isUnderWindowsFamily())
      {
         if (isBatchInBackground())
         {
            System.out.println(errmsg);
         }
         else
         {
            // TODO: In Windows the native message box should be displayed here.
            System.out.print(errmsg);
         }
      }
      else
      {
         System.out.println(errmsg);
      }
   }
   
   /**
    * Displays error message box with title in bach mode.
    *
    * @param    masterMsg
    *           Text of the message to display.
    * @param    title
    *           The title for the message box to display.
    */
   public void messageBox(String masterMsg, String title)
   {
      if (PlatformHelper.isUnderWindowsFamily())
      {
         if (isBatchInBackground())
         {
            System.out.println(title + ":" + masterMsg);
         }
         else
         {
            // TODO: In Windows the native message box should be displayed here.
            System.out.print(title + ":" + masterMsg);
         }
      }
      else
      {
         System.out.println(title + ":" + masterMsg);
      }
   }
   
   /**
    * Implements pause processing in bach mode.
    */
   public void pause()
   {
      // In batch mode do noting.
      return;
   }

   /**
    * Stops the associated client session by generating UnstoppableExitException.
    */
   public void stopBatchSession()
   {
      if (client != null)
      {
         client.stopBatchSession();
      }
   }
}
