/*
** Module   : BatchDriver.java
** Abstract : Batch mode based driver.
**
** Copyright (c) 2013-2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 EVL 20131104 Created initial version.
** 002 EVL 20140114 The key reader should return -2(EOF) in regular batch mode. Also we need to
**                  separate processing for background and foreground mode simulation. The native
**                  library loading has been moved to central location.
**                  See the ClientCore class.
** 002 EVL 20140224 The batch driver does not need to initialize console in Windows.  The Windows
**                  console modifies the screen in a way not acceptable for batch driver. Also we
**                  do not need to clear the screen in batch mode.  So the initChui() nas been
**                  removed.  The new method terminalType() returning empty string has been added
**                  to batch driver.  Both Linux and Windows 4GL return empty string for TERMINAL
**                  function.
*/

package com.goldencode.p2j.ui.client.chui.driver.batch;

import java.util.*;

import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.ui.chui.*;
import com.goldencode.p2j.ui.client.*;
import com.goldencode.p2j.ui.client.driver.*;
import com.goldencode.p2j.ui.client.event.*;
import com.goldencode.p2j.ui.client.chui.driver.console.*;
import com.goldencode.p2j.util.*;
import com.goldencode.util.*;

/**
 * The dummy console pseudo batch driver.  The underlying implementation is used to provide i/o
 * handling for batch mode.
 */
public class BatchDriver
extends ConsoleDriver
implements ScreenDriver
{
   /** Reference to batch low-level driver. */
   private BatchHelper batchHelper = BatchHelper.getDefaultBatchHelper();

   /** Direct to screen output primitives. */
   private BatchPrimitives batchOut;

   /** Flag indicating whether we need to sumulate foreground or background execution. */
   private boolean isBackground = false;
   
   /**
    * Constructor. 
    */
   public BatchDriver()
   {
      super();
   }

   /**
    * Generate a beep signal. 
    * <p>
    * Implementation deoends on native layer and (in turn) from particular 
    * terminal. If terminal does not support this feature or sound in terminal
    * is turned off, there will be no audible feedback to user.
    */
   @Override
   public void beep()
   {
      batchHelper.outputOperation();
   }

   /**
    * Clear screen.
    */
   @Override
   public void clear()
   {
      if (batchOut != null)
      {
         if (batchOut.isActive())
         {
            batchHelper.outputOperation();
         }
      }
      else
      {
         super.clear();
      }
   }

   /**
    * Get instance of {@link OutputPrimitives} which provides direct access to 
    * required functionality of attached terminal.
    */
   @Override
   public OutputPrimitives getPrimitives()
   {
      return batchOut;
   }

   /**
    * Check if terminal supports colors. 
    * <p>
    * This implementation also checks if colors are enabled in low-level 
    * driver. If they are not (and usually this is so), then application 
    * will run in B/W mode even on color terminal.
    */
   @Override
   public boolean hasColors()
   {
      batchHelper.inputOperation();
      return false;
   }

   /**
    * Checks if the driver is running in batch mode.
    * 
    * @return  <code>true</code> for batch mode, <code>false</code> otherwise.
    * 
    */
   public boolean inBatchMode()
   {
      return true;
   }

   /**
    * Checks if the current driver is running in background mode.
    * 
    * @return  <code>true</code> for background mode, <code>false</code> otherwise.
    * 
    */
   public boolean inBackgroundMode()
   {
      return isBackground;
   }

   /**
    * Sets the current running mode flag to simulate background mode.
    * 
    * @param   background
    *          <code>true</code> for background mode, <code>false</code> otherwise.
    */
   public void setBackgroundMode(boolean background)
   {
      isBackground = background;
   }

   /**
    * Read the key from terminal and translate it into P2J internal 
    * representation. Translation details are in {@link KeyProcessor} class. 
    * Mouse events (which also returned from native layer) are filtered out 
    * because mouse is not supported.  
    */
   @Override
   public int readKey()
   {
      if (batchOut != null)
      {
         if (batchOut.isActive())
         {
             // We are under Windows or Linux foreground mode
            if (PlatformHelper.isUnderWindowsFamily() || !isBackground)
            {
               batchOut.deActivate();
               // In Windows we have to generate the error and stop client.
               batchHelper.inputOperation();
            }
            return -2;
         }
         else
         {
            // Otherwise pass control to ConsoleDriver key handler 
            return super.readKey();
         }
      }
      else
      {
         return super.readKey();
      }
   }

   /**
    * Reset screen mode.
    */
   @Override
   public void resetMode()
   {
      batchHelper.outputOperation();
   }

   /**
    * Resume terminal after {@link #suspend()} call.
    */
   @Override
   public void resume()
   {
      if (batchOut != null)
      {
         if (batchOut.isActive())
         {
            batchHelper.outputOperation();
         }
      }
      else
      {
         super.resume();
      }
   }

   /**
    * Suspend terminal and release it for use by other applications.
    */
   @Override
   public void suspend()
   {
      if (batchOut != null)
      {
         if (batchOut.isActive())
         {
            batchHelper.outputOperation();
         }
      }
      else
      {
         super.suspend();
      }
   }

   /**
    * Get terminal type string.
    */
   @Override
   public String terminalType()
   {
      return "";
   }
   
   /**
    * Initialize driver.
    */
   @Override
   public void init()
   {
      // the 4GL outputs a Progress logo, copyright notice and version at the startup of batch 
      // modr when run in the foreground (and just the version when run in the background); we are
      // deliberately NOT duplicating these outputs for these reasons:
      // 1. they have no value for application/user processing or functionality
      // 2. generating that output would confuse people as to who owns this technology
      // 3. these values changes from version to version anyway, so it is very unlikely that an
      //    app would depend upon the output      
      batchOut = new BatchPrimitives(batchHelper);
   } 
}
