/*
 ** Module   : FileChecker.java
 ** Abstract : Java implementation of for detecting the file type attribute 
 *             list of OS-DIR
 **
 ** Copyright (c) 2013-2014, Golden Code Development Corporation.
 ** ALL RIGHTS RESERVED. Use is subject to license terms.
 **
 **           Golden Code Development Corporation
 **                      CONFIDENTIAL
 **
 ** -#- -I- --Date-- ----------------Description-----------------
 ** 001 CS  20130103 Created initial version for Implement OS-DIR support as
 **                  an INPUT stream task.
 ** 002 CS  20130325 Added getFileCreationTime native function for task 1613
 **                  finish FILE-INFO system handle support
 **                  Changed logic of getFileType to handle logic for UNIX and 
 **                  windows.
 ** 002 EVL 20140117 The native library call is performing now in dedicated place and only once.
 **                  See the ClientCore class.
 ** 003 EVL 20140221 Add native call to check if the standard console output is redirected to the
 **                  file by command line option.
 **/
package com.goldencode.p2j.util;

import java.io.File;

import com.goldencode.util.*;

/**
 * Utility class that uses JNI call to get the bitfield representing the 
 * status type of the given file path parameter and return the equivalent
 * Progress OS-DIR status.
 * <p>
 * The following codes are implemented:
 * <p>
 * <b>PRIMARY TYPES</b>
 * <ul>
 *    <li><b>F</b> - Regular file.
 *    <li><b>D</b> - Directory.
 *    <li><b>S</b> - Special Device
 *    <li><b>X</b> - Unknown type
 * </ul>
 * <p>
 * <b>SECONDARY (ADDITIVE) TYPES</b>
 * <ul>
 *    <li><b>H</b> - Hidden file.
 *    <li><b>L</b> - Symbolic link.
 *    <li><b>P</b> - Pipe File.
 * </ul>
 * <p>
 * One of the main types will always appear, and can also be accompanied by 
 * one or more of the secondary types.
 * <p>
 * In some cases the additive status can only be added to a specific main type
 * Ex: the Progress behavior for hidden attribute -> only hidden file are 
 * marked as hidden, hidden directories are only marked as directory.
 * <p>
 * Because the file system may be different from an OS to another (some
 * filetypes are Unix specific) results may be yielded differently, 
 * however this is handled by the native code
 *
 */
public class FileChecker
{
   /** Status code for error */
   private static final int ERROR_TYPE =-1;

   /** Status bitfield code for unknown */
   private static final int UNKNOWN_TYPE = 0;

   /** Status bitfield code for directory */
   private static final int DIRECTORY_TYPE = 1<<0;    // 000001

   /** Status bitfield code for file */
   private static final int FILE_TYPE = 1<<1;         // 000010

   /** Status bitfield code for symbolic link */
   private static final int SYMBOLIC_LINK_TYPE = 1<<2; //000100

   /** Status bitfield code for devices */
   private static final int DEVICE_TYPE = 1<<3;       // 001000

   /** Status bitfield code for pipes */
   private static final int PIPE_TYPE = 1<<4;         // 010000

   /** Status bitfield code for hidden */
   private static final int HIDDEN_TYPE = 1<<5;       // 100000
   
   /** Status code for UNIX */
   private static final int IS_UNIX = 1<<6;
   
   /** Status code for WINDOWS */
   private static final int IS_WINDOWS = 1<<7;

   /** Cache of console redirection flag for current session. */
   private static Boolean consoleRedirected = null;
   
   /**
    * Native utility method that will return a bitfield code representing the
    * status of the file.
    * 
    * @param   filePath
    *          The String representing the path of the file for
    *          which we want to calculate the status code.
    * 
    * @return  The status bitfield code of the file.
    */
   public native static int checkFileType(String filePath);
   
   /**
    * Native utility method that will return the creation date of a file 
    * represented as milliseconds from year 1970.  .
    * 
    * @param   filePath
    *          The String representing the path of the file for
    *          which we want to calculate the creation time.
    * 
    * @return  The file creation time in milliseconds from 1970 or -1 if 
    *          operation has failed , (real data cannot be -1 since precision
    *          for returned date is at the level of seconds).
    */
   public native static long getFileCreationTime(String filePath);

   /**
    * This method will call the native code handling the file type detection,
    * interpret the bitfield result and return the status in Progress format.
    * 
    * @param  filePath
    *         The String representing the path of the file for
    *         which we want to calculate the status code.
    *         
    * @return The composed status of the file.
    */
   public static String getFileType(String filePath)
   {
      StringBuilder sb = new StringBuilder();

      int val = checkFileType(filePath);
       if (val == -1)
      {
         // TODO This error condition could not be duplicated in Progress
         String errmsg = "Error encountered while getting file status of: " + 
                         filePath;
         ErrorManager.recordOrThrowError(-1, errmsg);
      }
      // cover windows behavior
      if ((val & IS_WINDOWS) != 0)
      {
         // primary type checking
         if ((val & FILE_TYPE) != 0)
         {
            sb.append('F');

            // hidden attribute is only applied to files on windows
            if ((val & HIDDEN_TYPE) != 0)
            {
               sb.append('H');
            }
         }
         else if ((val & DIRECTORY_TYPE) != 0)
         {
            sb.append('D');
         }
         else if ((val & DEVICE_TYPE) != 0)
         {
            sb.append('S');
         }
         else
         {
            // the special case of UNKNOWN_TYPE
            return "X";
         }

         // secondary
         if ((val & SYMBOLIC_LINK_TYPE) != 0)
         {
            // do nothing (windows behavior)
         }
         if ((val & PIPE_TYPE) != 0)
         {
            // do nothing (windows behavior)
         }
      }
      // cover unix behavior
      if ((val & IS_UNIX) != 0)
      {
         if ((val & HIDDEN_TYPE) != 0)
         {
            sb.append('H');
         }
         if ((val & SYMBOLIC_LINK_TYPE) != 0)
         {
            sb.append('L');
         }
         if ((val & FILE_TYPE) != 0)
         {
            sb.append('F');
         }
         else if ((val & DIRECTORY_TYPE) != 0)
         {
            sb.append('D');
         }
         else if ((val & DEVICE_TYPE) != 0)
         {
            sb.append('S');
         }
         else
         {
            // the special case of UNKNOWN_TYPE
            sb.append("X");
         }
         if ((val & PIPE_TYPE) != 0)
         {
            sb.append('P');
         }
      }
      return sb.toString();      
   }
   
   /**
    * Checks if the current STDOUT handle is redirected to the file by OS output redirection
    * command line option(">").
    *
    * @return   <code>true</code> if the current console output is redirected 
    *           <code>false</code> otherwise.
    */
   public static native boolean isStdoutRedirected();

   /**
    * Checks if the current console session is redirected to the file by OS output redirection
    * command line option(">").
    *
    * @return   <code>true</code> if the current console output is redirected 
    *           <code>false</code> otherwise.
    */
   public static synchronized boolean isConsoleRedirected()
   {
      if (consoleRedirected == null)
      {
         consoleRedirected = new Boolean(isStdoutRedirected());
      }
      
      return consoleRedirected.booleanValue();
   }
}
