/*
** Module   : filesys.c
** Abstract : implementation for detection of symbolic links, non-FIFO pipes
**            and special devices (used by com.goldencode.p2j.util.FileChecker)
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --------------------------------Description---------------------------------
** 001 CS  20130104 First version for OS-DIR support as an INPUT stream task.
** 002 CS  20130115 Task 1613: finish FILE-INFO system handle support (added a native function
**                  for getting the file creation date time).
** 003 EVL 20130329 Made changes to compile as a Windows DLL.
** 004 GES 20131029 Cleaned up possible memory leaks where we weren't relasing our reference to
**                  the string on the Java heap. Made code formatting improvements.
** 005 EVL 20140221 Adding callto check if the STDOUT is redirected in command line.
*/ 
#include "filesys.h"

#include "com_goldencode_p2j_util_FileChecker.h"
#include "com_goldencode_p2j_util_FileSystemDaemon.h"

/**
 * Utility method to check the type of the given file.
 * <p>
 * There are 4 main types a file can be: Directory , File , Device and Unknown. In addition to
 * the 4 main types there are 3 additive stats (they can be added to the main type). For example,
 * FH has both FILE and HIDDEN flags marked and DL has both DIRECTORY and LINK flags.
 * <p>
 * In some cases the additive status can only be added to a specific main type.  For example,
 * in Progress only hidden files are marked as hidden, hidden directories are only marked as
 * directory.
 *
 * @param    env 
 *           pointer to a structure that provides access to JNI environment
 * @param    jc 
 *           reference to the Java class who called this function
 *           (from static context)
 * @param    fname 
 *           The file name to be checked passed from FileChecker.checkFileType()
 *
 * @return   An bitfield representing the status codes for the given pathname.
 */
JNIEXPORT jint JNICALL 
       Java_com_goldencode_p2j_util_FileChecker_checkFileType(JNIEnv  *env,
                                                              jclass  jc, 
                                                              jstring fname) 
{
   const char *file_path = (*env)->GetStringUTFChars(env, fname, NULL);
   
   if (file_path == NULL) 
   {
      return -1; /* OutOfMemoryError already thrown */
   }
   
   int status = UNIX;
   int fd     = open_native(file_path, O_RDONLY);
   
   if (fd < 0)
   {
      // free the string on the Java heap
      (*env)->ReleaseStringUTFChars(env, fname, file_path);
      
     return -1;
   }
   
   struct stat st;
   
   // Use fstat to detect files and directories or links that point to files
   // and directories as files or directories
   if (fstat(fd, &st)) 
   { 
      // close the file 
      close(fd);
      
      // free the string on the Java heap
      (*env)->ReleaseStringUTFChars(env, fname, file_path);
      
      return -1; 
   }
   
   // check if regular file
   if (S_ISREG(st.st_mode)) 
   {
      // File status is standalone status
      status |= FILE;
   }
   
   // extract the filename from the path
   // replicate the Unix 4GL bug in which upper dir appears as hidden
   // by nontreating .. special case when checking for hidden.
   char *filename = (strrchr(file_path, '/'))+1;
   
   // in unix hidden files don't have a property that determines if
   // they are hidden instead a hidden file will start with '.'
   //check if first letter is '.'
   if (*filename == '.')
   {
      // hidden status is additive status
      status |= HIDDEN;
   }
   
   // check if file is directory
   if (S_ISDIR(st.st_mode)) 
   { 
      status |= DIR;
   }
   
   // check for fifo pipe
   if (S_ISFIFO(st.st_mode))
   {
      // Pipe code status is additive but may not detected as file
      // File code status is also added if
      if (!(status & FILE))
      {
         status |= FILE;
      }
      status |= PIPE;
   }
   
   // close the file 
   close(fd);
   
   // get the pratform specific sym link status
   status = getSymLinkStatus(env, file_path, status);
   
   // free the string on the Java heap
   (*env)->ReleaseStringUTFChars(env, fname, file_path);
   
   return status;  
}

/**
 * Utility method that will return the creation date of a file 
 * represented as milliseconds from year 1970.
 * 
 * @param    env 
 *           pointer to a structure that provides access to JNI environment
 * @param    jc 
 *           reference to the Java class who called this function
 *           (from static context)
 * @param    fname 
 *           The file name to be checked passed from 
 *           FileChecker.getFileCreationTime.
 *           
 * @return   A wrapped integer representing the  file creation time in 
 *           milliseconds from 1970 or -1 if operation has failed , 
 *           (real data cannot be -1 since precision
 *           for returned date is at the level of seconds).
 */
JNIEXPORT jlong JNICALL 
   Java_com_goldencode_p2j_util_FileChecker_getFileCreationTime(JNIEnv  *env,
                                                                jclass  jc, 
                                                                jstring fname) 
{
   const char *file_path = (*env)->GetStringUTFChars(env, fname, NULL);
   
   if (file_path == NULL) 
   {
      return -1; // OutOfMemoryError already thrown
   }
   
   struct stat st;
   
   int rc = stat(file_path, &st);
   
   (*env)->ReleaseStringUTFChars(env, fname, file_path);
   
   if (rc != 0) 
   {
      return -1; 
   }
   
   // the return variable must be long long otherwise it will truncate
   // on Linux/UNIX the only date that is kept is last modify time
   long long m = (long long) st.st_ctime;
   return m * 1000L;
}

/**
 * Utility method that will return the value of the system environment variable with a given
 * name. 
 * 
 * @param    env 
 *           pointer to a structure that provides access to JNI environment
 * @param    jc 
 *           reference to the Java class who called this function
 *           (from static context)
 * @param    vname 
 *           The name of the environment variable to get.
 *           
 * @return   A string representing the system anvironment variable if exists or NULL pointer if 
 *           the varable is unknown.
 */
JNIEXPORT jstring JNICALL 
   Java_com_goldencode_p2j_util_FileSystemDaemon_getEnvironmentValue(JNIEnv  *env,
                                                                     jclass  jc, 
                                                                     jstring vname) 
{
   const char* env_var = (*env)->GetStringUTFChars(env, vname, NULL);
   
   char* result = NULL;
   
   if (env_var != NULL)
   {
      if (strlen(env_var) > 0)
      {
         result = getenv(env_var);
      }
      
      (*env)->ReleaseStringUTFChars(env, vname, env_var);
   }
   
   return (result != NULL) ? (*env)->NewStringUTF(env, result) : NULL;
}

/**
 * Checks if the current STDOUT handle is redirected to the file by command line option.
 *
 * @param    env 
 *           pointer to a structure that provides access to JNI environment
 * @param    jc 
 *           reference to the Java class who called this function
 *           (from static context)
 *
 * @return   <code>true</code> if STDOUT is redirected <code>false</code> otherwise.
 */
JNIEXPORT jboolean JNICALL
   Java_com_goldencode_p2j_util_FileChecker_isStdoutRedirected(JNIEnv *env, jclass  jc)
{
   struct stat st;
   
   return (fstat(STDOUT_FILENO, &st) == 0) && // Means success call to fstat
          (S_ISCHR(st.st_mode) == 0) &&       // The output is not character device
          (S_ISREG(st.st_mode) != 0);         // but regular file
}

